@@ -167,8 +167,11 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data)
 		return NULL;
 	}
 
-	/* Return the file. */
-	*data = g_realloc (buffer, total_bytes_read);
+	/* Return the file null-terminated. */
+	buffer = g_realloc (buffer, total_bytes_read + 1);
+	buffer[total_bytes_read] = '\0';
+	*data = buffer;
+
 	mimetype = gnome_vfs_get_mime_type_for_data (*data, total_bytes_read);
 
 	return mimetype;
@@ -790,11 +793,11 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 }
 
 static TotemPlParserResult
-totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
-		gpointer data)
+totem_pl_parser_add_asf_reference_parser (TotemPlParser *parser,
+		const char *url, gpointer data)
 {
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
-	char *contents, **lines, *ref;
+	char *contents, **lines, *ref, *split_char;
 	int size;
 
 	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
@@ -803,7 +806,13 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 	contents = g_realloc (contents, size + 1);
 	contents[size] = '\0';
 
-	lines = g_strsplit (contents, "\n", 0);
+	if (strstr(contents,"\x0d") == NULL) {
+		split_char = "\n";
+	} else {
+		split_char = "\x0d\n";
+	}
+
+	lines = g_strsplit (contents, split_char, 0);
 	g_free (contents);
 
 	ref = read_ini_line_string (lines, "Ref1", FALSE);
@@ -813,7 +822,7 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 		return totem_pl_parser_add_asx (parser, url, data);
 	}
 
-	/* change http to mms, thanks Microsoft */
+	/* change http to mmsh, thanks Microsoft */
 	if (g_str_has_prefix (ref, "http") != FALSE)
 		memcpy(ref, "mmsh", 4);
 
@@ -826,6 +835,38 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 	return retval;
 }
 
+static TotemPlParserResult
+totem_pl_parser_add_asf_parser (TotemPlParser *parser,
+		const char *url, gpointer data)
+{
+	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	char *contents, *ref;
+	int size;
+
+	if (g_str_has_prefix (data, "ASF ") == FALSE) {
+		return totem_pl_parser_add_asf_reference_parser (parser, url, data);
+	}
+
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+
+	if (size <= 4) {
+		g_free (contents);
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
+
+	/* Skip 'ASF ' */
+	ref = contents + 4;
+	if (g_str_has_prefix (ref, "http") != FALSE) {
+		memcpy(ref, "mmsh", 4);
+		totem_pl_parser_add_one_url (parser, ref, NULL);
+		retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
+	g_free (contents);
+	return retval;
+}
+
 static TotemPlParserResult
 totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 {
@@ -848,8 +889,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	contents[size] = '\0';
 
 	/* figure out whether we're a unix pls or dos pls */
-	if (strstr(contents,"\x0d") == NULL)
-	{
+	if (strstr(contents,"\x0d") == NULL) {
 		split_char = "\n";
 	} else {
 		split_char = "\x0d\n";
@@ -858,8 +898,13 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	lines = g_strsplit (contents, split_char, 0);
 	g_free (contents);
 
-	/* [parser] */
-	if (g_ascii_strncasecmp (lines[0], "[playlist]",
+	/* [playlist] */
+	i = 0;
+	while (lines[i] != NULL && strcmp (lines[i], "") == 0)
+		i++;
+	
+	if (lines[i] == NULL 
+			|| g_ascii_strncasecmp (lines[i], "[playlist]",
 				(gsize)strlen ("[playlist]")) != 0)
 		goto bail;
 
@@ -932,7 +977,7 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		xmlNodePtr parent, const char *pl_title)
 {
 	xmlNodePtr node;
-	char *title, *url;
+	char *title, *url, *fullpath;
 	gboolean retval = FALSE;
 
 	title = NULL;
@@ -960,22 +1005,19 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		return FALSE;
 	}
 
-	if (strstr (url, "://") != NULL || url[0] == '/') {
-		totem_pl_parser_add_one_url (parser, url,
-				title ? title : pl_title);
-		retval = TRUE;
-	} else {
-		char *fullpath;
-
+	if (strstr ((char *)url, "://") == NULL && url[0] != '/') {
 		fullpath = g_strdup_printf ("%s/%s", base, url);
-		/* .asx files can contain references to other .asx files */
-		if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS)
-			totem_pl_parser_add_one_url (parser, fullpath,
-					title ? title : pl_title);
-
-		g_free (fullpath);
+	} else {
+		fullpath = g_strdup ((char *)url);
 	}
 
+	/* .asx files can contain references to other .asx files */
+	if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		totem_pl_parser_add_one_url (parser, fullpath,
+				(char *)title ? (char *)title : pl_title);
+	}
+
+	g_free (fullpath);
 	g_free (title);
 	g_free (url);
 
@@ -1200,7 +1242,8 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 	if (data != NULL &&
 		(g_str_has_prefix (data, "[Reference]") == FALSE
 		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0
-		 && strstr (data, "<ASX") == NULL)) {
+		 && strstr (data, "<ASX") == NULL)
+		 && g_str_has_prefix (data, "ASF ") == FALSE) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
@@ -1273,9 +1316,10 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	MediaType type;
 	GList *list, *l;
 	GnomeVFSResult res;
-	char *media_url;
 
 	if (parser->priv->recurse_level == 1) {
+		char *media_url;
+
 		type = totem_cd_detect_type_from_dir (url, &media_url, NULL);
 		if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR) {
 			if (media_url != NULL) {
@@ -1295,7 +1339,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 	l = list;
 
 	while (l != NULL) {
-		char *name, *str, *fullpath;
+		char *name, *fullpath;
 		GnomeVFSFileInfo *info = l->data;
 
 		if (info->name != NULL && (strcmp (info->name, ".") == 0
@@ -1304,22 +1348,13 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			continue;
 		}
 
-		if (g_utf8_validate (info->name, -1, NULL) == FALSE)
-			name = g_strdup (info->name);
-		else
-			name = gnome_vfs_escape_string (info->name);
-		str = g_build_filename (url, info->name, NULL);
-
-		if (strstr (str, "://") != NULL && str[0] == '/')
-			fullpath = str + 1;
-		else
-			fullpath = str;
+		name = gnome_vfs_escape_string (info->name);
+		fullpath = g_strconcat (url, "/", name, NULL);
+		g_free (name);
 
 		if (totem_pl_parser_parse_internal (parser, fullpath) != TOTEM_PL_PARSER_RESULT_SUCCESS)
 			totem_pl_parser_add_one_url (parser, fullpath, NULL);
 
-		g_free (str);
-		g_free (name);
 		l = l->next;
 	}
 
@@ -1464,6 +1499,8 @@ totem_pl_parser_parse (TotemPlParser *parser, const char *url,
 {
 	g_return_val_if_fail (TOTEM_IS_PL_PARSER (parser), TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	g_return_val_if_fail (url != NULL, TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_return_val_if_fail (strstr (url, "://") != NULL,
+			TOTEM_PL_PARSER_RESULT_UNHANDLED);
 
 	parser->priv->recurse_level = 0;
 	parser->priv->fallback = fallback;
