@@ -84,6 +84,11 @@
 
 #define ZOOM_IN_OFFSET 1
 #define ZOOM_OUT_OFFSET -1
+#define ZOOM_UPPER 200
+#define ZOOM_RESET 100
+#define ZOOM_LOWER 10
+
+#define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
 
 #define BVW_VBOX_BORDER_WIDTH 1
 
@@ -110,6 +115,7 @@ static void show_controls (Totem *totem, gboolean was_fullscreen);
 static gboolean totem_is_fullscreen (Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
+static void popup_timeout_remove (Totem *totem);
 
 static void
 long_action (void)
@@ -162,6 +168,8 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
+	popup_timeout_remove (totem);
+
 #ifndef HAVE_GTK_ONLY
 	totem_gromit_clear (TRUE);
 #endif /* !HAVE_GTK_ONLY */
@@ -308,7 +316,7 @@ totem_action_play (Totem *totem)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
@@ -335,7 +343,7 @@ totem_action_seek (Totem *totem, double pos)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
@@ -434,22 +442,34 @@ totem_action_play_pause (Totem *totem)
 	}
 }
 
-void
-totem_action_fullscreen_toggle (Totem *totem)
+static gboolean
+window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
+		Totem *totem)
 {
-	if (totem_is_fullscreen (totem) != FALSE)
-	{
+	if (event->changed_mask != GDK_WINDOW_STATE_FULLSCREEN) {
+		return FALSE;
+	}
+
+	if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
+		totem_action_save_size (totem);
+		update_fullscreen_size (totem);
+		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
+		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
+
+		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
+			totem_scrsaver_disable (totem->scr);
+
+		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
+		show_controls (totem, FALSE);
+	} else {
 		GtkWidget *item;
 
 		popup_hide (totem);
 		bacon_video_widget_set_fullscreen (totem->bvw, FALSE);
-		gtk_window_unfullscreen (GTK_WINDOW(totem->win));
 		bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
 		totem_scrsaver_enable (totem->scr);
 
-		totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
-
 		item = glade_xml_get_widget
 			(totem->xml, "tmw_show_controls_menu_item");
 
@@ -462,17 +482,18 @@ totem_action_fullscreen_toggle (Totem *totem)
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
 			show_controls (totem, TRUE);
 		}
-	} else {
-		totem_action_save_size (totem);
-		update_fullscreen_size (totem);
-		bacon_video_widget_set_fullscreen (totem->bvw, TRUE);
-		bacon_video_widget_set_show_cursor (totem->bvw, FALSE);
-		gtk_window_fullscreen (GTK_WINDOW(totem->win));
-		if (bacon_video_widget_is_playing (totem->bvw) != FALSE)
-			totem_scrsaver_disable (totem->scr);
+	}
 
-		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
-		show_controls (totem, FALSE);
+	return FALSE;
+}
+
+void
+totem_action_fullscreen_toggle (Totem *totem)
+{
+	if (totem_is_fullscreen (totem) != FALSE) {
+		gtk_window_unfullscreen (GTK_WINDOW (totem->win));
+	} else {
+		gtk_window_fullscreen (GTK_WINDOW (totem->win));
 	}
 }
 
@@ -709,7 +730,7 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		{
 			char *msg, *disp;
 
-			disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+			disp = totem_uri_escape_for_display (totem->mrl);
 			msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 			g_free (disp);
 			totem_action_error (msg, err->message, totem);
@@ -814,7 +835,7 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), totem->mrl);
 		g_free (disp);
 
@@ -826,19 +847,60 @@ totem_action_seek_relative (Totem *totem, int off_sec)
 	}
 }
 
+static void
+totem_action_zoom (Totem *totem, int zoom)
+{
+	GtkWidget *item;
+
+	if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER)
+		return;
+
+	bacon_video_widget_set_zoom (totem->bvw, zoom);
+
+	if (zoom == ZOOM_UPPER) {
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_in_menu_item");
+		gtk_widget_set_sensitive (item, FALSE);
+	} else if (zoom == ZOOM_LOWER) {
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_out_menu_item");
+		gtk_widget_set_sensitive (item, FALSE);
+	} else if (zoom == ZOOM_RESET) {
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_reset_menu_item");
+		gtk_widget_set_sensitive (item, FALSE);
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_in_menu_item");
+		gtk_widget_set_sensitive (item, TRUE);
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_out_menu_item");
+		gtk_widget_set_sensitive (item, TRUE);
+	} else {
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_in_menu_item");
+		gtk_widget_set_sensitive (item, TRUE);
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_out_menu_item");
+		gtk_widget_set_sensitive (item, TRUE);
+		item = glade_xml_get_widget
+			(totem->xml, "tmw_zoom_reset_menu_item");
+		gtk_widget_set_sensitive (item, TRUE);
+	}
+}
+
 static void
 totem_action_zoom_relative (Totem *totem, int off_pct)
 {
 	int zoom;
 
 	zoom = bacon_video_widget_get_zoom (totem->bvw);
-	bacon_video_widget_set_zoom (totem->bvw, zoom + off_pct);
+	totem_action_zoom (totem, zoom + off_pct);
 }
 
 static void
 totem_action_zoom_reset (Totem *totem)
 {
-	bacon_video_widget_set_zoom (totem->bvw, 100);
+	totem_action_zoom (totem, 100);
 }
 
 void
@@ -1086,10 +1148,14 @@ on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
 
 	uri = egg_recent_item_get_uri (item);
 
+	totem_signal_block_by_data (totem->playlist, totem);
+
 	end = totem_playlist_get_last (totem->playlist);
 	playlist_changed = totem_playlist_add_mrl (totem->playlist, uri, NULL);
 	egg_recent_model_add_full (totem->recent_model, item);
 
+	totem_signal_unblock_by_data (totem->playlist, totem);
+
 	if (playlist_changed)
 	{
 		char *mrl;
@@ -1165,9 +1231,9 @@ on_error_event (BaconVideoWidget *bvw, char *message,
 		play_pause_set_label (totem, STATE_STOPPED);
 
 	if (fatal == FALSE) {
-		totem_action_error (_("An error occured"), message, totem);
+		totem_action_error (_("An error occurred"), message, totem);
 	} else {
-		totem_action_error_and_exit (_("An error occured"),
+		totem_action_error_and_exit (_("An error occurred"),
 				message, totem);
 	}
 }
@@ -1770,6 +1836,9 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	GtkWidget *menubar, *controlbar, *statusbar, *item, *bvw_vbox, *widget;
 	int width = 0, height = 0;
 
+	if (totem->bvw == NULL)
+		return;
+
 	menubar = glade_xml_get_widget (totem->xml, "tmw_menubar");
 	controlbar = glade_xml_get_widget (totem->xml, "tmw_controls_vbox");
 	statusbar = glade_xml_get_widget (totem->xml, "tmw_statusbar");
@@ -2070,7 +2139,7 @@ commit_hide_skip_to (GtkDialog *dialog, gint response, Totem *totem)
 	{
 		char *msg, *disp;
 
-		disp = gnome_vfs_unescape_string_for_display (totem->mrl);
+		disp = totem_uri_escape_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not seek in '%s'."), disp);
 		g_free (disp);
 		totem_action_stop (totem);
@@ -2228,8 +2297,10 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 		break;
 	}
 
-	if (handled != FALSE)
+	if (handled != FALSE &&
+			gtk_window_is_active (GTK_WINDOW (totem->win))) {
 		on_video_motion_notify_event (NULL, NULL, totem);
+	}
 }
 
 #ifdef HAVE_REMOTE
@@ -2332,10 +2403,12 @@ playlist_shuffle_toggle_cb (TotemPlaylist *playlist, gboolean shuffle, Totem *to
 static void
 update_fullscreen_size (Totem *totem)
 {
-	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
+	GdkScreen *screen;
+
+	screen = gtk_window_get_screen (GTK_WINDOW (totem->win));
+	gdk_screen_get_monitor_geometry (screen,
 			gdk_screen_get_monitor_at_window
-			(gdk_screen_get_default (),
-			 totem->win->window),
+			(screen, totem->win->window),
 			&totem->fullscreen_rect);
 }
 
@@ -2351,6 +2424,8 @@ move_popups (Totem *totem)
 	int control_width, control_height;
 	int exit_width, exit_height;
 
+	update_fullscreen_size (totem);
+
 	gtk_window_get_size (GTK_WINDOW (totem->control_popup),
 			&control_width, &control_height);
 	gtk_window_get_size (GTK_WINDOW (totem->exit_popup),
@@ -2368,23 +2443,20 @@ move_popups (Totem *totem)
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.width - control_width,
-				totem->fullscreen_rect.height
-				- control_height);
+				totem->fullscreen_rect.height + totem->fullscreen_rect.y - control_height);
 	} else {
 		gtk_window_move (GTK_WINDOW (totem->exit_popup),
-				totem->fullscreen_rect.width - exit_width,
+				totem->fullscreen_rect.width + totem->fullscreen_rect.x - exit_width,
 				totem->fullscreen_rect.y);
 		gtk_window_move (GTK_WINDOW (totem->control_popup),
 				totem->fullscreen_rect.x,
-				totem->fullscreen_rect.height
-				- control_height);
+				totem->fullscreen_rect.height + totem->fullscreen_rect.y - control_height);
 	}
 }
 
 static void
 size_changed_cb (GdkScreen *screen, Totem *totem)
 {
-	update_fullscreen_size (totem);
 	move_popups (totem);
 }
 
@@ -2428,7 +2500,7 @@ on_mouse_click_fullscreen (GtkWidget *widget, Totem *totem)
 {
 	popup_timeout_remove (totem);
 
-	totem->popup_timeout = g_timeout_add (2000,
+	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
 		(GSourceFunc) popup_hide, totem);
 }
 
@@ -2460,7 +2532,7 @@ on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event,
 	gtk_widget_show_all (totem->control_popup);
 	bacon_video_widget_set_show_cursor (totem->bvw, TRUE);
 
-	totem->popup_timeout = g_timeout_add (5000,
+	totem->popup_timeout = g_timeout_add (FULLSCREEN_POPUP_TIMEOUT,
 			(GSourceFunc) popup_hide, totem);
 	totem->popup_in_progress = FALSE;
 
@@ -3046,6 +3118,8 @@ totem_callback_connect (Totem *totem)
 	g_object_notify (G_OBJECT (totem->win), "is-active");
 	g_signal_connect_swapped (G_OBJECT (totem->win), "notify",
 			G_CALLBACK (popup_hide), totem);
+	g_signal_connect (G_OBJECT (totem->win), "window-state-event",
+			G_CALLBACK (window_state_event_cb), totem);
 
 	/* Screen size and Theme changes */
 	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
@@ -3411,11 +3485,10 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	g_set_application_name (_("Totem Movie Player"));
-
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
+		g_set_application_name (_("Totem Movie Player"));
 		totem_action_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
 	}
 
@@ -3432,6 +3505,7 @@ main (int argc, char **argv)
 			GNOME_PARAM_NONE);
 #endif /* HAVE_GTK_ONLY */
 
+	g_set_application_name (_("Totem Movie Player"));
 	gnome_vfs_init ();
 
 	if ((gc = gconf_client_get_default ()) == NULL)
